SystemOrganization addCategory: #'EjerciciosPoo-TusLibros'!

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:30'!
dummyMerchantProcessorInterface

	^MerchantProcessorInterfaceSimulator new initializeWith: [ ]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/4/2013 19:32'!
expiredCreditCard

	^CreditCard new initializeWith: 123 expirationDate: (Date today subtractDays: 120) andOwner: 'Juan Gomez'! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/4/2013 19:43'!
nonExpiredCreditCard

	^CreditCard new initializeWith: 123 expirationDate: (Date today addDays: 120) andOwner: 'Juan Gomez'! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:30'!
stolenCardMerchantProcessorInterface

	^MerchantProcessorInterfaceSimulator new initializeWith: [ Error signal: MerchantProcessorInterfaceSimulator stolenCreditCardErrorMessage ]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:09'!
test01CartShouldBeEmptyAfterCreation

	| cart priceList |
	
	priceList := Dictionary new.
	cart := Cart new initializeWith: priceList.
	self assert: cart isEmpty! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:09'!
test02CartShouldNotBeEmptyAfterAddingABook

	| cart book priceList |
	
	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	
	self deny: cart isEmpty! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:10'!
test03CartShouldNotAllowToAddABookWhichIsNotInTheCatalog

	| cart book priceList |

	book := Book new initializeWith: 123.
	priceList := Dictionary new.	
	cart := Cart new initializeWith: priceList.
	
	self should: [ cart add: book ] raise: Error withExceptionDo: [ :anError | 
		self assert: anError messageText = Cart itemNotInCatalogErrorMessage.
		self assert: cart isEmpty
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:10'!
test04AbookThatIsAddedToTheCartIsInTheCart

	| cart priceList book |

	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	
	self assert: (cart includes: book)! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 20:59'!
test05CannotCheckoutAnEmptyCart

	| cart priceList cashier creditCard salesBookSize |

	priceList := Dictionary new.
	cart := Cart new initializeWith: priceList.
	creditCard := CreditCard new.
	cashier := Cashier new initializeWith: self dummyMerchantProcessorInterface.
	salesBookSize := cashier salesBook size.
	
	self should: [ cashier checkout: cart with: creditCard ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = Cashier emptyCartErrorMessage.
		self assert: salesBookSize = cashier salesBook size
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:14'!
test06CheckoutWithOneItemGeneratesASaleWithTheSamePriceAsTheItem

	| cart priceList cashier book creditCard |

	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	cashier := Cashier new initializeWith: self dummyMerchantProcessorInterface.
	creditCard := self nonExpiredCreditCard.
	
	self assert: ((cashier checkout: cart with: creditCard) total = (priceList at: book isbn))! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:00'!
test07CheckoutWithManyItemsGeneratesASaleWithTheSumOfTheItemsPricesAsTotal

	| cart priceList cashier aBook anotherBook creditCard |

	aBook := Book new initializeWith: 123.
	anotherBook := Book new initializeWith: 321.
	priceList := Dictionary newFromPairs: (Array with: aBook isbn with: 5 with: anotherBook isbn with: 8).
	cart := Cart new initializeWith: priceList.
	cart add: aBook.
	cart add: anotherBook.
	cashier := Cashier new initializeWith: self dummyMerchantProcessorInterface.
	creditCard := self nonExpiredCreditCard.
	
	self assert: ((cashier checkout: cart with: creditCard) total = ((priceList at: aBook isbn) + (priceList at: anotherBook isbn)) )! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:01'!
test08CheckoutOfAnItemGeneratesASaleThatIncludesTheItem

	| cart priceList cashier book creditCard |

	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	cashier := Cashier new initializeWith: self dummyMerchantProcessorInterface.
	creditCard := self nonExpiredCreditCard.
	
	self assert: ((cashier checkout: cart with: creditCard) includesBook: book isbn)! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:02'!
test09ChangesInPriceListDontAffectOlderSalesPrices

	| cart priceList cashier book saleTotal creditCard |

	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	cashier := Cashier new initializeWith: self dummyMerchantProcessorInterface.
	creditCard := self nonExpiredCreditCard.
	saleTotal := (cashier checkout: cart with: creditCard) total.
	priceList at: book isbn put: 8.
	
	self assert: (saleTotal = cashier salesBook last total )! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:31'!
test10CannotCheckoutWithExpiredCreditCard

	| cart priceList cashier book creditCard salesBookSize |

	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	cashier := Cashier new initializeWith: MerchantProcessorInterfaceSimulator new.
	salesBookSize := cashier salesBook size.
	creditCard := self expiredCreditCard.
	
	self should: [ cashier checkout: cart with: creditCard ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = CreditCard expiredCardErrorMessage.
		self assert: salesBookSize = cashier salesBook size
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:30'!
test11CannotCheckoutWithStolenCreditCard

	| cart priceList cashier book creditCard salesBookSize |

	book := Book new initializeWith: 123.
	priceList := Dictionary newFromPairs: (Array with: book isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: book.
	cashier := Cashier new initializeWith: self stolenCardMerchantProcessorInterface.
	salesBookSize := cashier salesBook size.
	creditCard := self nonExpiredCreditCard.
	
	self should: [ cashier checkout: cart with: creditCard ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = MerchantProcessorInterface stolenCreditCardErrorMessage.
		self assert: salesBookSize = cashier salesBook size
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:38'!
test12InvalidClientIdCannotCreateCart

	| restInterface |
	
	restInterface := RestInterface new.
	self should: [ restInterface createCartForClientId: 1 withPassword: 'abc' ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = RestInterface invalidClientIdErrorMessage.
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:38'!
test13ValidClientIdWithInvalidPasswordCannotCreateCart

	| restInterface |
	
	restInterface := RestInterface new.
	restInterface addClientId: 1 withPassword: 'abc'.
	self should: [ restInterface createCartForClientId: 1 withPassword: 'abz' ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = RestInterface invalidPasswordErrorMessage.
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:39'!
test14ValidClientGetsCartIdAfterCreateCart

	| restInterface |
	
	restInterface := RestInterface new.
	restInterface addClientId: 1 withPassword: 'abc'.
	self assert: (restInterface isInCurrentCarts: (restInterface createCartForClientId: 1 withPassword: 'abc'))! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:57'!
test15CartCreatedByValidClientIsEmptyAfterCreation

	| restInterface |
	
	restInterface := RestInterface new.
	restInterface addClientId: 1 withPassword: 'abc'.
	self assert: (restInterface listCart: (restInterface createCartForClientId: 1 withPassword: 'abc')) isEmpty! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:59'!
test16ValidClientCannotAddBookThatIsNotInThePriceList

! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:59'!
test17BookAddedByValidClientIsIncludedInHisCart

! !

Object subclass: #Book
	instanceVariableNames: 'isbn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Book methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:47'!
initializeWith: anIsbn

	isbn := anIsbn! !

!Book methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:47'!
isbn

	^isbn! !

Object subclass: #Cart
	instanceVariableNames: 'booksInCart priceList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 5/29/2013 20:38'!
itemNotInCatalogErrorMessage

	^'El item no se encuentra en el catlogo'! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:11'!
add: aBook

	^(priceList includesKey: aBook isbn) ifTrue: [ booksInCart add: aBook ] ifFalse: [ Error signal: self class itemNotInCatalogErrorMessage ]! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:20'!
booksInCart

	^booksInCart! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 5/29/2013 20:51'!
includes: aBook

	^booksInCart includes: aBook! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:11'!
initializeWith: aPriceList

	booksInCart := Bag new.
	priceList := aPriceList! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 5/29/2013 17:39'!
isEmpty

	^booksInCart isEmpty! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:11'!
priceList

	^priceList! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 21:37'!
totalAmountToCheckout

	| total |
	
	total := 0.
	booksInCart do: [ :book | total := total + (priceList at: book isbn) ].
	^total

! !

Object subclass: #Cashier
	instanceVariableNames: 'salesBook mpi'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 5/29/2013 21:06'!
emptyCartErrorMessage

	^'El carrito est vacio'! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:02'!
checkout: aCart with: aCreditCard

	| newSale bookWithPrice |

	aCart isEmpty ifTrue: [ Error signal: self class emptyCartErrorMessage ] ifFalse: [
		(aCreditCard expiredOn: (Date today)) ifTrue: [ Error signal: aCreditCard class expiredCardErrorMessage ].
		mpi debit: aCart totalAmountToCheckout from: aCreditCard.
		newSale := Sale new.
		aCart booksInCart do: [ :book | bookWithPrice := Association key: book isbn value: (aCart priceList at: book isbn). newSale add: bookWithPrice ].
		salesBook add: newSale.
		^newSale
	]! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:49'!
initializeWith: aMerchantProcessorInterface

	salesBook := OrderedCollection new.
	mpi := aMerchantProcessorInterface! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 20:24'!
salesBook

	^salesBook! !

Object subclass: #CreditCard
	instanceVariableNames: 'number expirationDate owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!CreditCard class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 20:42'!
expiredCardErrorMessage

	^'La tarjeta se encuentra vencida'! !

!CreditCard methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:04'!
expiredOn: aDate

	^expirationDate < aDate! !

!CreditCard methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:21'!
initializeWith: aNumber expirationDate: anExpirationDate andOwner: anOwner

	number := aNumber.
	expirationDate := anExpirationDate.
	owner := anOwner! !

Object subclass: #MerchantProcessorInterface
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!MerchantProcessorInterface class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:28'!
stolenCreditCardErrorMessage

	^'La tarjeta es robada'! !

!MerchantProcessorInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/4/2013 19:15'!
debit: anAmount from: aCreditCard

	^self subclassResponsibility! !

MerchantProcessorInterface subclass: #MerchantProcessorInterfaceSimulator
	instanceVariableNames: 'debitFromAction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!MerchantProcessorInterfaceSimulator methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 20:16'!
debit: anAmount from: aCreditCard

	^debitFromAction value! !

!MerchantProcessorInterfaceSimulator methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 20:51'!
initializeWith: aDebitFromAction

	debitFromAction := aDebitFromAction! !

Object subclass: #RestInterface
	instanceVariableNames: 'validUsers currentCarts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!RestInterface class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:36'!
invalidClientIdErrorMessage

	^'El id de cliente es invlido'! !

!RestInterface class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/9/2013 20:08'!
invalidPasswordErrorMessage

	^'El password es invlido'! !

!RestInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:35'!
addClientId: aClientId withPassword: aPassword

	^validUsers at: aClientId put: aPassword! !

!RestInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:54'!
createCartForClientId: aClientId withPassword: aPassword

	| cartId |
	
	(self validateClientId: aClientId withPassword: aPassword) ifTrue: [ 
		cartId := self newCartId.
		currentCarts at: cartId put: (Cart new initializeWith: Dictionary new).
		^cartId
	]ifFalse: [ 
		Error signal: self class invalidPasswordErrorMessage
	]! !

!RestInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/9/2013 20:18'!
initialize

	validUsers := Dictionary new.
	currentCarts := Dictionary new! !

!RestInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:28'!
isInCurrentCarts: aCartId

	^currentCarts includesKey: aCartId! !

!RestInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:51'!
listCart: aCartId

	| cart |
	
	cart := currentCarts at: aCartId.
	^cart booksInCart sortedElements! !

!RestInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:29'!
newCartId

	^Random new nextInt: 999999! !

!RestInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:36'!
validateClientId: aClientId withPassword: aPassword

	^(validUsers at: aClientId ifAbsent: [ Error signal: self class invalidClientIdErrorMessage ] ) = aPassword! !

Object subclass: #Sale
	instanceVariableNames: 'items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:40'!
add: anItem

	items add: anItem.! !

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 18:26'!
includesBook: anIsbn

	^items includesKey: anIsbn! !

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:38'!
initialize

	items := Dictionary new.! !

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:44'!
total

	| total |
	
	total := 0.
	items do: [ :item | total := total + item value ].
	^total! !
